clear, clc, mode(0), close(winsid())
//// Řízení s modelem $y_t=b_0u_t+a_1y_{t-1}+b_1u_{t-1}+a_2y_{t-2}+b_2u_{t-2}+k+e_t$ a penalizační funkcí $J_t=(y_t-s_t)^2\omega+(u_t-u_{t-1})^2\lambda$

load data8.sod;
y=rychlost;
u=plyn;
nd=size(y,1);    //počet dat pro odhad




om=1; la=.1;             // penalizace 
Om=diag([om la 0 la 0]);                 // penalizační matice
Om(2,4)=-la; Om(4,2)=-la;


nh=100;                   // interval řízení

setpoint=2;
select setpoint
case 1
    s=90*ones(1,(nd+nh)); // setpoint - doporučená rychlost
case 2   
    s=5*sin(19*(1:(nd+nh))/nd)+90;
end


///// Offline odhad parametrů z apriorních dat
ndi=nd;   // počet apriorních dat  
t=3:ndi;                       
Y=y(t);                      // vektor výstupů
X=[u(t) y(t-1) u(t-1) y(t-2) u(t-2)  ones(ndi-2,1)];   //matice regresních vektorů
theta_odhad=inv(X'*X)*X'*Y;           // bodové odhady regresních koeficientů
r_odhad=Y'*(Y-X*theta_odhad)/nd;      //bodový odhad rozptylu


//// převod do stavového tvaru
///regresní koeficienty
b0=theta_odhad(1);
a1=theta_odhad(2);
b1=theta_odhad(3);
a2=theta_odhad(4);
b2=theta_odhad(5);
k=theta_odhad(6);
sd=sqrt(r_odhad); // směrodatná odchylka


M=[a1 b1 a2 b2 k; 
   0  0  0  0  0;
   1  0  0  0  0;
   0  1  0  0  0;
   0  0  0  0  1];           
N=[b0 1  0  0  0]'; 





S=list();          
R=0; // počáteční podmínky pro dynamické programování 
// výpočet optimálního řízení - dynamické programování
for t=nh:-1:1
  Om(1,$)=-om*s(t); Om($,1)=-om*s(t); Om($,$)=om*s(t)**2;
  T=R+Om;
  A=N'*T*N;
  B=N'*T*M;
  C=M'*T*M;
  S(t)=inv(A)*B;
  R=C-S(t)'*A*S(t);
end

//použití optimálního řízení
yo(1)=y(nd-1); yo(2)=y(nd-2); /// počáteční podmínky
uo(1)=u(nd-1); uo(2)=u(nd-2);

for t=3:nh
  x=[yo(t-1) uo(t-1) yo(t-2) uo(t-2) 1]'; /// stav
  uo(t)=-S(t)*x;     // řídící vstup
  ps=[uo(t) yo(t-1) uo(t-1) yo(t-2) uo(t-2) 1]; //regresní vektor
  // generování výstupu 
  yo(t)=ps*theta_odhad+sd*rand(1,1,'n');   
end





// výsledky
// data offline + s optimálním řízením
y=[y; yo]; 
u=[u; uo];

plot(y)
plot(u,'g')
plot(s,'r:')
title ('Řízení s doporučenou rychlostí','fontsize',5)
legend('rychlost','plyn','doporučená rychlost',4);
ylabel('Data','fontsize',5)
xlabel('Čas','fontsize',5)
